@@ -56,6 +56,7 @@ getFMSY <- function(x, Marray, hs, Mat_age, Wt_age, R0, V, maxage, nyears,
 #' @param Wt_age internal parameter
 #' @param R0 internal parameter
 #' @param V internal parameter
+#' @param retA internal parameter
 #' @param maxage internal parameter
 #' @param nyears internal parameter
 #' @param proyears internal parameter
@@ -67,16 +68,18 @@ getFMSY <- function(x, Marray, hs, Mat_age, Wt_age, R0, V, maxage, nyears,
 #' 
 #' @keywords internal
 #' @export getFMSY2	
-getFMSY2 <- function(x, M_ageArray, hs, Mat_age, Wt_age, R0, V, maxage, nyears, 
+getFMSY2 <- function(x, M_ageArray, hs, Mat_age, Wt_age, R0, V, retA, maxage, nyears, 
     proyears, Spat_targ, mov, SRrel, aR, bR, Control=1) {
-    opt <- optimize(projOpt_cpp, log(c(0.001, 5)),
-		Mc = M_ageArray[x, ,nyears], hc = hs[x], Mac = Mat_age[x, ], Wac = Wt_age[x, , nyears], R0c = R0[x], 
-        Vc = V[x, ,nyears], nyears = nyears, maxage = maxage, movc = mov[x, , ], Spat_targc = Spat_targ[x],
-        SRrelc = SRrel[x], aRc = aR[x, ], bRc = bR[x, ], proyears = proyears, Control=Control)
+    opt <- optimize(projOpt_cpp, log(c(0.001, 5)), Mc = M_ageArray[x, ,nyears], hc = hs[x], 
+                    Mac = Mat_age[x, ], Wac = Wt_age[x, , nyears], R0c = R0[x], 
+                    Vc = V[x, ,nyears], retAc=retA[x,,nyears], nyears = nyears, 
+                    maxage = maxage, movc = mov[x, , ], 
+                    Spat_targc = Spat_targ[x],  SRrelc = SRrel[x], 
+                    aRc = aR[x, ], bRc = bR[x, ], proyears = proyears, Control=Control)
 	  MSY <- -opt$objective 
 	  MSYs <- projOpt_cpp(lnIn = opt$minimum, Mc = M_ageArray[x, ,nyears], hc = hs[x], 
 	                      Mac = Mat_age[x, ], Wac = Wt_age[x, , nyears], R0c = R0[x], 
-	                      Vc = V[x, ,nyears], nyears = nyears, maxage = maxage, movc = mov[x, , ], 
+	                      Vc = V[x, ,nyears], retAc=retA[x,,nyears], nyears = nyears, maxage = maxage, movc = mov[x, , ], 
 	                      Spat_targc = Spat_targ[x], SRrelc = SRrel[x], aRc = aR[x, ], 
 	                      bRc = bR[x, ], proyears = proyears, Control=2)
     SSB_MSY <- MSYs[1]				
@@ -340,6 +343,7 @@ getFref <- function(x, Marray, Wt_age, Mat_age, Perr, N_s, SSN_s, Biomass_s,
 #' @param VBiomass_s internal parameter
 #' @param SSB_s internal parameter
 #' @param Vn internal parameter
+#' @param retAn internal parameter
 #' @param hs internal parameter
 #' @param R0a internal parameter
 #' @param nyears internal parameter
@@ -356,18 +360,18 @@ getFref <- function(x, Marray, Wt_age, Mat_age, Perr, N_s, SSN_s, Biomass_s,
 #' @keywords internal
 #' @export getFref2
 getFref2 <- function(x, M_ageArray, Wt_age, Mat_age, Perr, N_s, SSN_s, Biomass_s, 
-    VBiomass_s, SSB_s, Vn, hs, R0a, nyears, proyears, nareas, maxage, mov, 
-    SSBpR, aR, bR, SRrel, Spat_targ) {
-    	
-    opt <- optimize(doprojPI_cpp, log(c(0.001, 5)), Mmat = M_ageArray[x, , (nyears + 1):(nyears + proyears)], 
-	  Wac = Wt_age[x, , (nyears + 1):(nyears + proyears)], Mac = Mat_age[x, ], 
-	    Pc = Perr[x, (maxage+nyears):(maxage-1+nyears + proyears)], N_c = N_s[x, , 1,], 
-		SSN_c = SSN_s[x, , 1, ], Biomass_c = Biomass_s[x, , 1, ], 
-		VBiomass_c = VBiomass_s[x, , 1, ], SSB_c = SSB_s[x, , 1, ], Vc = Vn[x, , ], 
-		hc = hs[x], R0ac = R0a[x, ], proyears, nareas, maxage, movc = mov[x, , ], 
-		SSBpRc = SSBpR[x], aRc = aR[x, ], bRc = bR[x, ], SRrelc = SRrel[x], 
-        Spat_targc = Spat_targ[x])
-    
+                     VBiomass_s, SSB_s, Vn, retAn, hs, R0a, nyears, proyears, nareas, maxage, mov, 
+                     SSBpR, aR, bR, SRrel, Spat_targ) {
+  
+  opt <- optimize(doprojPI_cpp, log(c(0.001, 5)), Mmat = M_ageArray[x, , (nyears + 1):(nyears + proyears)], 
+                  Wac = Wt_age[x, , (nyears + 1):(nyears + proyears)], Mac = Mat_age[x, ], 
+                  Pc = Perr[x, (maxage+nyears):(maxage-1+nyears + proyears)], N_c = N_s[x, , 1,], 
+                  SSN_c = SSN_s[x, , 1, ], Biomass_c = Biomass_s[x, , 1, ], 
+                  VBiomass_c = VBiomass_s[x, , 1, ], SSB_c = SSB_s[x, , 1, ], Vc = Vn[x, , ], 
+                  retAc=retAn[x,,], hc = hs[x], R0ac = R0a[x, ], proyears, nareas, maxage, movc = mov[x, , ], 
+                  SSBpRc = SSBpR[x], aRc = aR[x, ], bRc = bR[x, ], SRrelc = SRrel[x], 
+                  Spat_targc = Spat_targ[x])
+   
     return(-opt$objective)
 		  
 }
@@ -478,15 +482,11 @@ doprojPI <- function(lnF, Mvec, Wac, Mac, Pc, N_c, SSN_c, Biomass_c, VBiomass_c,
 		# if (y==2) print(fishdist)
 		
         Z_P[AYR] <- FM_P[AYR] + Mvec[Y]
-        CNtemp <- N_P[, y, ] * exp(Z_P[, y, ]) * (1 - exp(-Z_P[, y, ])) * 
-            (FM_P[, y, ]/Z_P[, y, ])
-        CB_P[y] <- sum(Biomass_P[, y, ] * exp(Z_P[, y, ]) * (1 - exp(-Z_P[, 
-            y, ])) * (FM_P[, y, ]/Z_P[, y, ]))
+        CNtemp <- N_P[, y, ] * exp(Z_P[, y, ]) * (1 - exp(-Z_P[, y, ])) * (FM_P[, y, ]/Z_P[, y, ])
+        CB_P[y] <- sum(Biomass_P[, y, ] * exp(Z_P[, y, ]) * (1 - exp(-Z_P[, y, ])) * (FM_P[, y, ]/Z_P[, y, ]))
         
-        CNtemp <- (FM_P[, y, ]/Z_P[, y, ] * N_P[, y, ] * (1 - exp(-Z_P[, 
-            y, ])))
-        CB_P[y] <- sum(FM_P[, y, ]/Z_P[, y, ] * Biomass_P[, y, ] * (1 - 
-            exp(-Z_P[, y, ])))
+        CNtemp <- (FM_P[, y, ]/Z_P[, y, ] * N_P[, y, ] * (1 - exp(-Z_P[, y, ])))
+        CB_P[y] <- sum(FM_P[, y, ]/Z_P[, y, ] * Biomass_P[, y, ] * (1 - exp(-Z_P[, y, ])))
         
         # temp <- sum(CNtemp*Wac[AY]) print(c(CB_P[y], temp))
         
@@ -661,8 +661,10 @@ SelectFun <- function(i, SL0.05, SL1, MaxSel, maxlens, Lens) {
 #' @param age95 age at 95 percent maturity
 #' @keywords internal
 #' @export getroot
-getroot <- function(X, ageM, age95) 
-  uniroot(getSlopeFun, interval = c(1e-04, 5), age50 = ageM[X], age95 = age95[X])$root
+getroot <- function(X, ageM, age95) {
+  uniroot(getSlopeFun, interval = c(1e-08, 8), age50 = ageM[X], age95 = age95[X])$root
+}
+  
   
 #' Internal function to calculate slope
 #'
